"""
The MIT License (MIT)

Copyright (c) 2020 - Present, PythonistaGuild

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:
The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""

from __future__ import annotations

import datetime
from typing import Optional

import aiohttp

from .http import HTTPClient
from .paste import File, Paste
from .utils import require_authentication


__all__ = ("Client",)


class Client:
    __slots__ = ("http",)

    def __init__(self, *, token: Optional[str] = None, session: Optional[aiohttp.ClientSession] = None) -> None:
        self.http: HTTPClient = HTTPClient(token=token, session=session)

    async def close(self) -> None:
        """|coro|

        Closes the internal HTTP session and this client.
        """
        if self.http._session:
            await self.http._session.close()

    async def create_paste(
        self,
        *,
        filename: str,
        content: str,
        password: Optional[str] = None,
        expires: Optional[datetime.datetime] = None,
    ) -> Paste:
        """|coro|

        Create a single file paste on mystb.in.

        Parameters
        -----------
        filename: :class:`str`
            The filename to create.
        content: :class:`str`
            The content of the file you are creating.
        password: Optional[:class:`str`]
            The password of the paste, if any.
        expires: Optional[:class:`datetime.datetime`]
            When the paste expires, if any.

        Returns
        --------
        :class:`mystbin.Paste`
            The paste that was created.
        """
        file = File(filename=filename, content=content)
        data = await self.http._create_paste(file=file, password=password, expires=expires)
        return Paste._from_data(data)

    async def create_multifile_paste(
        self, *, files: list[File], password: Optional[str] = None, expires: Optional[datetime.datetime] = None
    ) -> Paste:
        """|coro|

        Create a paste with multiple files on mystb.in.

        Parameters
        ------------
        files: list[:class:`mystbin.File`]
            A list of files to create on mystbin.
        password: Optional[:class:`str`]
            The password for this paste, if any.
        expires: Optional[:class:`datetime.datetime`]
            When this paste expires, if any.

        Returns
        --------
        :class:`mystbin.Paste`
            The paste that was created.
        """
        data = await self.http._create_paste(files=files, password=password, expires=expires)
        return Paste._from_data(data)

    @require_authentication
    async def delete_paste(self, paste_id: str, /) -> None:
        """|coro|

        Delete a paste.

        Parameters
        -----------
        paste_id: :class:`str`
            The paste to delete.
        """
        await self.http._delete_pastes(paste_ids=[paste_id])

    @require_authentication
    async def delete_pastes(self, paste_ids: list[str], /) -> None:
        """|coro|

        Delete multiple pastes.

        Parameters
        -----------
        paste_ids: list[:class:`str`]
            The pastes to delete.
        """
        await self.http._delete_pastes(paste_ids=paste_ids)

    async def get_paste(self, paste_id: str, *, password: Optional[str] = None) -> Paste:
        """|coro|

        Fetch a paste.

        Parameters
        -----------
        paste_id: :class:`str`
            The paste id to fetch.
        password: Optional[:class:`str`]
            The password of the paste, if any.
        """
        data = await self.http._get_paste(paste_id=paste_id, password=password)
        return Paste._from_data(data)

    # @overload
    # async def edit_paste(self, paste_id: str, *, new_content: str, new_filename: ..., new_expires: ...) -> None:
    #     ...

    # @overload
    # async def edit_paste(self, paste_id: str, *, new_content: ..., new_filename: str, new_expires: ...) -> None:
    #     ...

    # @overload
    # async def edit_paste(
    #     self, paste_id: str, *, new_content: ..., new_filename: ..., new_expires: datetime.datetime
    # ) -> None:
    #     ...

    # @overload
    # async def edit_paste(self, paste_id: str, *, new_content: str, new_filename: str, new_expires: ...) -> None:
    #     ...

    # @overload
    # async def edit_paste(
    #     self, paste_id: str, *, new_content: str, new_filename: ..., new_expires: datetime.datetime
    # ) -> None:
    #     ...

    # @overload
    # async def edit_paste(
    #     self, paste_id: str, *, new_content: ..., new_filename: str, new_expires: datetime.datetime
    # ) -> None:
    #     ...

    # @overload
    # async def edit_paste(
    #     self, paste_id: str, *, new_content: str, new_filename: str, new_expires: datetime.datetime
    # ) -> None:
    #     ...

    # @require_authentication
    # async def edit_paste(
    #     self,
    #     paste_id: str,
    #     *,
    #     new_content: Optional[str] = MISSING,
    #     new_filename: Optional[str] = MISSING,
    #     new_expires: Optional[datetime.datetime] = MISSING,
    # ) -> None:
    #     await self.http._edit_paste(paste_id, new_content=new_content, new_filename=new_filename, new_expires=new_expires)

    @require_authentication
    async def get_user_pastes(self, *, limit: int = 100) -> list[Paste]:
        """|coro|

        Get all pastes belonging to the current authenticated user.

        Parameters
        -----------
        limit: :class:`int`
            The amount of pastes to fetch. Defaults to ``100``.

        Returns
        --------
        list[:class:`Paste`]
            The pastes that were fetched.
        """
        data = await self.http._get_my_pastes(limit=limit)

        return [Paste._from_data(x) for x in data]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mystbin', 'mystbin.types']

package_data = \
{'': ['*']}

modules = \
['py']
install_requires = \
['aiohttp>=3.7.4,<4.0.0']

extras_require = \
{'docs': ['sphinx>=4.0.0,<5.0.0', 'sphinxcontrib-trio', 'furo'],
 'requests': ['requests>=2.24.0,<3.0.0']}

entry_points = \
{'console_scripts': ['version = mystbin.__main__:show_version']}

setup_kwargs = {
    'name': 'mystbin-py',
    'version': '5.0.3',
    'description': 'A small simple wrapper around the mystb.in API.',
    'long_description': '<div align="center">\n    <h1>Mystbin.py!</h1>\n    <a href=\'https://mystbinpy.readthedocs.io/en/latest/?badge=latest\'>\n        <img src=\'https://readthedocs.org/projects/mystbinpy/badge/?version=latest\' alt=\'Documentation Status\' />\n    </a>\n    <a href=\'https://github.com/PythonistaGuild/mystbin.py/workflows/Code%20Linting\'>\n        <img src=\'https://github.com/PythonistaGuild/mystbin.py/workflows/Code%20Linting/badge.svg?branch=main\' alt=\'Linting status\' />\n    </a>\n    <a href=\'https://github.com/PythonistaGuild/mystbin.py/workflows/Build\'>\n        <img src=\'https://github.com/PythonistaGuild/mystbin.py/workflows/Build/badge.svg\' alt=\'Build status\' />\n    </a>\n</div>\n<br>\n\nA small simple wrapper around the [Mystb.in](https://mystb.in/) API. API docs can be found [here](https://api.mystb.in/docs).\n\nDocumentation for this wrapper can be found [here](https://mystbinpy.readthedocs.io/en/stable/).\nIf you want the docs for the `main` branch, those can be found [here](https://mystbinpy.readthedocs.io/en/latest/).\n\n----------\n### Features\n\n- [x] - Creating pastes.\n- [ ] - Editing pastes.\n    - Pending design work.\n- [x] - Deleting pastes.\n- [x] - Getting pastes.\n- [ ] - User endpoints.\n- [ ] - Sync client.\n  - This one will take some time as I have no motivation to do it, but PRs are welcome if others want to do it.\n\n### Installation\nThis project will be on [PyPI](https://pypi.org/project/mystbin.py/) as a stable release, you can always find that there.\n\nInstalling via `pip`:\n```shell\npython -m pip install -U mystbin.py\n```\n\nInstalling from source:\n```shell\npython -m pip install git+https://github.com/PythonistaGuild/mystbin.py.git\n```\n\n### Usage examples\nSince the project is considered multi-sync, it will work in a sync/async environment, see the optional dependency of `requests` below.\n\n```py\n# async example - it will default to async\nimport mystbin\n\nmystbin_client = mystbin.Client()\n\npaste = await client.create_paste(filename="Hello.txt", content="Hello there!")\nstr(paste)\n>>> \'https://mystb.in/<your generated ID>\'\n\nget_paste = await mystbin_client.get_paste("<your generated ID>")\nget_paste.files[0].content\n>>> "Hello there!"\n\nget_paste.created_at\n>>> datetime.datetime(2020, 10, 6, 10, 53, 57, 556741)\n```\n\nOr if you want to create a paste with multiple files...\n```py\nimport mystbin\n\nfile = mystbin.File(filename="File1.txt", content="Hello there!")\nfile2 = mystbin.File(filename="test.py", content="print(\'hello!\')")\n\npaste = await client.create_multifile_paste(files=[file, file2])\n\nfor file in paste.files:\n    print(file.content)\n\n>>> "Hello there!"\n>>> "print(\'hello!\')"\n```\n\nIf you have any question please feel free to join the Pythonista Discord server:\n<div align="left">\n    <a href="https://discord.gg/RAKc3HF">\n        <img src="https://discordapp.com/api/guilds/490948346773635102/widget.png?style=banner2" alt="Discord Server"/>\n    </a>\n</div>\n',
    'author': 'AbstractUmbra',
    'author_email': 'Umbra@AbstractUmbra.dev',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/PythonistaGuild/mystbin.py',
    'packages': packages,
    'package_data': package_data,
    'py_modules': modules,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



"""
The MIT License (MIT)

Copyright (c) 2020 - Present, PythonistaGuild

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:
The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""

from __future__ import annotations

import datetime
from typing import TYPE_CHECKING, Any, Optional


if TYPE_CHECKING:
    from typing_extensions import Self

    from mystbin.types.responses import FileResponse, PasteResponse

__all__ = (
    "File",
    "Paste",
)


class File:
    """Represents a single file within a mystb.in paste.

    Attributes
    -----------
    filename: :class:`str`
        The file's name.
    content: :class:`str`
        The file's contents.
    lines_of_code: Optional[:class:`int`]
        The lines of code within the file.
    character_count: Optional[:class:`int`]
        The character count of the file.


    .. note::
        The ``lines_of_code`` and ``character_count`` come from the API and should not be provided by the user.
    """

    __slots__ = (
        "filename",
        "content",
        "lines_of_code",
        "character_count",
    )

    def __init__(
        self,
        *,
        filename: str,
        content: str,
        lines_of_code: Optional[int] = None,
        character_count: Optional[int] = None,
    ) -> None:
        self.filename: str = filename
        self.content: str = content
        self.lines_of_code: int = lines_of_code or content.count("\n")
        self.character_count: int = character_count or len(content)

    @classmethod
    def _from_data(cls, payload: FileResponse, /) -> Self:
        return cls(
            content=payload["content"],
            filename=payload["filename"],
            lines_of_code=payload["loc"],
            character_count=payload["charcount"],
        )

    def _to_dict(self) -> dict[str, Any]:
        ret: dict[str, Any] = {"content": self.content, "filename": self.filename}

        return ret


class Paste:
    """Represents a Paste object from mystb.in.

    Attributes
    -----------
    id: :class:`str`
        The ID of this paste.
    created_at: :class:`datetime.datetime`
        When this paste was created in UTC.
    expires: Optional[:class:`datetime.datetime`]
        When this paste expires, if at all.
    last_edited: Optional[:class:`datetime.datetime`]
        When this paste was last edited, if at all.
    files: list[:class:`mystbin.File`]
        The list of files within this Paste.
    views: Optional[:class:`int`]
        How many views this paste has had, if any.


    .. note::
        The ``last_edited``, ``expires`` and ``views`` attributes come from the API and are not user provided.
    """

    __slots__ = (
        "id",
        "author_id",
        "created_at",
        "expires",
        "files",
        "notice",
        "views",
        "last_edited",
    )

    def __init__(
        self,
        *,
        id: str,
        created_at: str,
        expires: Optional[str] = None,
        last_edited: Optional[str] = None,
        files: list[File],
        views: Optional[int] = None,
    ) -> None:
        self.id: str = id
        self.created_at: datetime.datetime = datetime.datetime.fromisoformat(created_at)
        self.expires: Optional[datetime.datetime] = datetime.datetime.fromisoformat(expires) if expires else None
        self.last_edited: Optional[datetime.datetime] = datetime.datetime.fromisoformat(last_edited) if last_edited else None
        self.files: list[File] = files
        self.views: Optional[int] = views

    def __str__(self) -> str:
        return self.url

    def __repr__(self) -> str:
        return f"<Paste id={self.id!r} files={len(self.files)}>"

    @property
    def url(self) -> str:
        return f"https://mystb.in/{self.id}"

    @classmethod
    def _from_data(cls, payload: PasteResponse, /) -> Self:
        files = [File._from_data(data) for data in payload["files"]]
        return cls(
            id=payload["id"],
            created_at=payload["created_at"],
            expires=payload["expires"],
            files=files,
            views=payload.get("views"),
            last_edited=payload.get("last_edited"),
        )
